import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';

class HttpUtils {
  // 单例创建本类
  factory HttpUtils() => _getInstance();

  static HttpUtils get instance => _getInstance();
  static HttpUtils _instance;
  static var GET = "get";
  static var POST = "post";
  static var DELETE = "delete";
  Dio _dio;
  var _tonken = 'eWRtaDp5ZG1o';

  // 初始化httputils工具类
  HttpUtils._internal() {
    // 初始化dio
    BaseOptions options = new BaseOptions(
      baseUrl: "http://192.168.1.156:9999",
      connectTimeout: 5000000,
      receiveTimeout: 3000000,
      contentType: Headers.formUrlEncodedContentType,
      headers: {"Authorization": "Basic $_tonken"},
      responseType: ResponseType.json,
    );
    _dio = new Dio(options);
    // 添加一个默认的cookie
    //_dio.interceptors.add(CookieManager(CookieJar()));
  }

  static HttpUtils _getInstance() {
    if (_instance == null) {
      _instance = HttpUtils._internal();
    }
    return _instance;
  }

  //get请求
  get(var action,
      {Map<String, dynamic> params,
      Function callBack,
      Function errorCallBack}) async {
    if (params == null) params = new Map();
    var data = await _request(GET, action,
        params: params, errorCallBack: errorCallBack, callBack: callBack);
    return data;
  }

  //post请求
  Future post(var action,
      {Map<String, dynamic> params,
      String contentType: Headers.jsonContentType,
      Function callBack,
      Function errorCallBack}) async {
    if (params == null) params = {};
    print('params===$params');
    var data = await _request(POST, action,
        params: params,contentType: contentType, errorCallBack: errorCallBack, callBack: callBack);
    print("postdata $data");
    return data;
  }

  //delete请求
  Future delete(var action,
      {Map<String, dynamic> params,
      Function callBack,
      Function errorCallBack}) async {
    if (params == null) params = new Map();
    var data = await _request(DELETE, action,
        params: params, errorCallBack: errorCallBack, callBack: callBack);
    return data;
  }

  static clean() {
    _instance = null;
  }

  _request(String method, String action,
      {Map<String, dynamic> params,
      String contentType,
      Function callBack,
      Function errorCallBack}) async {
    String errorMsg = "";
    int statusCode;
    Response response;
    var formData;
    // 设置请求数据格式
    if ("/auth/oauth/token" == action || Headers.formUrlEncodedContentType == contentType || '/auth/oauth/token?flag=true' == action) {
      _dio.options.contentType = Headers.formUrlEncodedContentType;
      formData = FormData.fromMap(params);
    } else if(Headers.jsonContentType == contentType){
      _dio.options.contentType = Headers.jsonContentType;
      formData = jsonEncode(params);
    } else if('multipart/form-data' == contentType) {
      _dio.options.contentType = 'multipart/form-data';
      formData = FormData.fromMap(params);
    }
    try {
      if (method == GET) {
        // get请求
        response = await _dio.get(action, queryParameters: params);
      } else if (method == POST) {
        // post请求
        response = await _dio.post(action, data: formData);
      } else if (method == DELETE) {
        // delete请求
        response = await _dio.delete(action, data: formData);
      }
      // http请求状态码
      statusCode = response.statusCode;
      //处理错误部分
      if (statusCode < 0) {
        errorMsg = "网络请求错误,状态码:" + statusCode.toString();
        _handError(errorCallBack, errorMsg);
      }
      if (callBack != null) {
        callBack(response);
//        if ('0000' == response.data['errCode']) {
//          callBack(response.data);
//          print("<net> response data:" + response.data.toString());
//        } else
//          _handError(errorCallBack, response.data['errMsg']);
      }
    } catch (exception) {
      _handError(errorCallBack, exception.toString());
    } finally {
      debugPrint("返回参数为: ${response.data?.toString()}");
      // 返回数据 即使失败也要返回null
      if ((response.data?.toString())?.isNotEmpty) {
        _tonken = response.data['access_token'];
        if (_tonken != null) {
          // 返回的token再次装载
          _dio.options.headers['Authorization'] = 'Bearer $_tonken';
        }
        return response.data;
      }
    }
  }

  //处理异常
  static void _handError(Function errorCallback, String errorMsg) {
    if (errorCallback != null) {
      errorCallback(errorMsg);
    }
    print("<net> errorMsg :" + errorMsg);
  }
}
